package WWW::Mechanize::Plugin::JavaScript;

use strict;   # :-(
use warnings; # :-(

use JE 0.015;
use Scalar::Util qw'weaken refaddr';

our $VERSION = '0.000002';

# Attribute constants (array indices)
sub mech() { 0 }
sub wind() { 1 } # window object

sub init { # expected to return a plugin object that the mech object will
           # use to communicate with the plugin.

	my ($package, $mech, @opts) = @_;

	my $self = bless [$mech], $package;
	weaken $self->[mech];
	# ~~~ Do we even need this ref to mech?

	my $scripter = sub {
		my($doc,$code,$url,$line,$inline) = @_;

		$code =~ s/^\s*<!--[^\cm\cj\x{2028}\x{2029}]*(?x:
		         )(?:\cm\cj?|[\cj\x{2028}\x{2029}])// if $inline;

#warn $code if $inline;
#warn $url unless $inline;
		
		my $w = $$self[wind] ||=
		 new WWW::Mechanize::Plugin::JavaScript::Window $mech,$doc;

		# ~~~ Do I need multiple window objects?

		# JE 0.015 (which we require) does not support multple
		# arguments for eval, but it doesn't mind them (it just
		# ignores them). Version 0.016 actually uses them.
		$w->eval($code, $url, $line);
		$@ and $mech->warn($@);
	};

	my $event_attribute_handler = sub {
		my($elem,$event_name,$code) = @_;

		my $w = $$self[wind] ||=
		    new WWW::Mechanize::Plugin::JavaScript::Window
		        $mech, $elem->ownerDocument;
		# ~~~ Do I need multiple window objects?

		# ~~~ JE's interface needs to be improved. This is a mess:
		my $func = $w->compile("(function(){ $code })")->execute(
			$w, bless [$w, $w->upgrade($elem)], 'JE::Scope');

		sub {
			my $event_obj = shift;
			my $ret = &$func($event_obj);
			$ret->typeof ne 'undefined' and !$ret and
				$event_obj->preventDefault;
			# ~~~ I need to change this logic for whichever
			#     event has it reversed (don't remember which
			#     it was; I'll have to look it up!).
		};
	};

	$mech->use_plugin(DOM =>
		script_handlers => {
		    default => $scripter,
		    qr/(?:^|\/)(?:x-)?(?:ecma|j(?:ava)?)script[\d.]*\z/i =>
				$scripter,
		},
		event_attr_handler => {
		    default => $event_attribute_handler,
		    qr/(?:^|\/)(?:x-)?(?:ecma|j(?:ava)?)script[\d.]*\z/i =>
				$event_attribute_handler,
		},
	);

	weaken $mech; # stop closures from preventing destruction

	$self;
}

# ~~~ we need to accept (alert => sub { ... }, confirm => sub { ... }) as
#     args
sub options {} # maybe later we can provide options for different JS
                   # engines


# ------------ WINDOW CLASS -----------------#

package WWW::Mechanize::Plugin::JavaScript::Window;

use Scalar::Util 'weaken';

our $VERSION = '0.000001';
our @ISA = 'JE';

sub new {
	my $self = SUPER::new{shift};
	my $mech = shift;

	weaken $mech;

	# ~~~ I need to finish all these class bindings.

	# Base class, so we don't need to add Node's methods to every
	# single class
	$self->bind_class(
		package => 'HTML::DOM::Node',
		name    => 'Node',
		methods => [qw/ insertBefore replaceChild removeChild
                          appendChild hasChildNodes cloneNode
		    addEventListener removeEventListener dispatchEvent /],
		props   => [qw/ nodeName nodeValue nodeType parentNode
                     childNodes firstChild lastChild previousSibling
                     nextSibling attributes ownerDocument /],
	);

	$self->bind_class(
		package => 'HTML::DOM',
		name    => 'HTMLDocument',
		methods => [qw/ createElement createDocumentFragment
		               createTextNode createComment
		             createCDATASection createProcessingInstruction
		          createAttribute createEntityReference
		      getElementsByTagName open close write writeln
		   getElementById getElementsByName /],
		props => [ qw/ doctype implementation
		                  documentElement title referrer
		                    domain URL body images applets
		                      links forms anchors cookies /],
		isa => 'Node',
	);

	$self->bind_class(
		package => 'HTML::DOM::Element',
		name    => 'HTMLElement',
		isa => 'Node',
	);

	$self->bind_class(
		package => 'HTML::DOM::Element::Body',
		name    => 'HTMLBodyElement',
		props => [qw/ aLink background bgColor link
		                           text vLink /],
		isa => 'HTMLElement',
	);

	$self->bind_class(
		package => 'HTML::DOM::NodeList',
		wrapper   => sub {
			WWW::Mechanize::Plugin::JavaScript::NodeList
			 ->new(@_);
		},
		name    => 'NodeList',
	);
	$self->new_function(NodeList => sub {
		die 'NodeList cannot be instantiated.';
	})->prop('prototype')
	  ->new_method(item => sub { shift->value->[shift] });
	

	$self->bind_class(
		package => 'HTML::DOM::Collection',
		wrapper   => sub {
			WWW::Mechanize::Plugin::JavaScript::HTMLCollection
			 ->new(@_);
		},
		name    => 'HTMLCollection',
	);
	my $hc_proto = $self->new_function(NodeList => sub {
		die 'NodeList cannot be instantiated.';
	})->prop('prototype');
	$hc_proto->new_method(item => sub { shift->value->[shift] });
	$hc_proto->new_method(namedItem => sub { shift->value->{shift} });


	# ~~~ I also need to finish all these properties

	$self->{document} = shift;

# ~~~	$self->{document}->prop({
#		name => cookie,
#		fetch => 
#	});

	$self->{navigator}{userAgent} = $mech->agent;
	$self->{window} = $self;
}

# ------------ NODE LIST CLASS -----------------#

package WWW::Mechanize::Plugin::JavaScript::NodeList;

our $VERSION = '0.000001';
our @ISA = 'JE::Object';


sub new {
	my ($w, $nl) = @_; # window, nodelist
	
	my $self = JE::Object::Array->new($w);
	$self->prototype($w->prop('NodeList')->prop('prototype'));
	$$$self{WMPDOM_nodelist} = $nl;
}

sub prop {
	my ($self, $name, $val) =  (shift, @_);
	my $guts = $$self;

	if ($name =~ /^(?:0|[1-9]\d*)\z/ and $name < 4294967295) {
		return $$guts{WMPDOM_nodelist}[$name];
	}
	$self->SUPER::prop(@_);
}

sub is_enum {
	my ($self,$name) = @_;
	if ($name =~ /^(?:0|[1-9]\d*)\z/ and $name < 4294967295) {
		return defined $$$self{WMPDOM_nodelist}[$name];
	}
	SUPER::is_enum $self $name;
}

sub keys {
	my $self = shift;
	0..$#{$$$self{WMPDOM_nodelist}}, SUPER::keys $self;
}

sub delete { !1 }

sub value { $${$_[0]}{WMPDOM_nodelist} };

sub exists {
	my ($self, $name) =  (shift, @_);
	my $guts = $$self;

	if ($name =~ /^(?:0|[1-9]\d*)\z/ and $name < 4294967295) {
		return defined $$guts{WMPDOM_nodelist}[$name];
	}
	$self->SUPER::exists(@_);
}

sub class { 'NodeList' }


# ------------ HTML COLLECTION CLASS -----------------#

package WWW::Mechanize::Plugin::JavaScript::HTMLCollection;
# (long package name!)

our $VERSION = '0.000001';
our @ISA = 'WWW::Mechanize::Plugin::JavaScript::NodeList';

# ~~~ finish this class


# ------------------ DOCS --------------------#

1;


=head1 NAME

WWW::Mechanize::Plugin::JavaScript - JavaScript plugin for WWW::Mechanize

=head1 VERSION

Version 0.001

B<WARNING:> This is a pre-alpha release. The API is subject to change 
without
notice. Some of the features have not
yet been implemented.

=head1 SYNOPSIS

  # etc. etc. etc.
  
=head1 DESCRIPTION

blah blah blah

=head1 REQUIREMENTS

perl (5.8.0 or later, I think; though it might work with 5.6)

WWW::Mechanize x.xx or later

HTML::DOM 0.003 or later

JE 0.015 or later

LWP::Simple

URI

=head1 BUGS

Need plenty of placeholders here :-)

=over 4

=item *

=item *

=item *

=item *

=item *

=item *

=item *

=item *

=item *

=item *

=item *

=item *

=item *

=item *

=item *

=back

=head1 AUTHOR

Father Chrysostomos <join '', Font::GlyphNames::name2str
qw[s p r o u t at c p a n period o r g]>

=head1 SEE ALSO

=over 4

=item -

L<WWW::Mechanize>

=item -

L<WWW::Mechanize::Plugin::DOM>

=item -

L<HTML::DOM>

=item -

L<JE>

=cut




